/*
 * $Header: /home/cvspublic/jakarta-commons-sandbox/resources/src/test/org/apache/commons/resources/impl/ResourcesBaseTestCase.java,v 1.6 2003/08/17 03:34:14 craigmcc Exp $
 * $Revision: 1.6 $
 * $Date: 2003/08/17 03:34:14 $
 *
 * ====================================================================
 *
 * The Apache Software License, Version 1.1
 *
 * Copyright (c) 1999-2002 The Apache Software Foundation.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution, if
 *    any, must include the following acknowlegement:
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowlegement may appear in the software itself,
 *    if and wherever such third-party acknowlegements normally appear.
 *
 * 4. The names "The Jakarta Project", "Commons", and "Apache Software
 *    Foundation" must not be used to endorse or promote products derived
 *    from this software without prior written permission. For written
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache"
 *    nor may "Apache" appear in their names without prior written
 *    permission of the Apache Group.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 *
 */


package org.apache.commons.resources.impl;


import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Locale;

import junit.framework.Test;
import junit.framework.TestSuite;

import org.apache.commons.resources.Resources;
import org.apache.commons.resources.ResourcesException;
import org.apache.commons.resources.ResourcesKeyException;


/**
 * <p>Unit tests for
 * <code>org.apache.commons.resources.impl.ResourcesBase</code>.
 * </p>
 *
 * @author Craig R. McClanahan
 * @version $Revision: 1.6 $ $Date: 2003/08/17 03:34:14 $
 */

public class ResourcesBaseTestCase extends ResourcesFactoryBaseTestCase {


    // ----------------------------------------------------- Instance Variables


    // The Resources instance to be tested
    protected Resources resources = null;


    // ----------------------------------------------------------- Constructors


    public ResourcesBaseTestCase(String name) {
        super(name);
    }


    // --------------------------------------------------- Overall Test Methods


    // Set up instance variables required by this test case
    public void setUp() throws Exception {
        super.setUp();
        resources = factory.getResources(NAME);
    }

    // Return the tests included in this test suite
    public static Test suite() {
        return (new TestSuite(ResourcesBaseTestCase.class));
    }

    // Tear down the instance variables required by this test case
    public void tearDown() {
        resources = null;
        super.tearDown();
    }


    // ------------------------------------------------ Individual Test Methods


    // Test retrieving Strings that should be inherited from parent Locales
    public void testInherit() throws Exception {

        Locale locale = null;

        locale = new Locale("en", "US");
        assertEquals("en_US inherit value",
                     "[test] INHERIT",
                     resources.getString("test.inherit", locale, null));

        locale = new Locale("fr", "FR");
        assertEquals("fr_FR inherit value",
                     "[test] INHERIT",
                     resources.getString("test.inherit", locale, null));

    }


    // Test retrieving all of the defined keys
    public void testKeys() throws Exception {

        // Pre-seed all of the keys we expect to find
        Locale locale = new Locale("en", "US");
        resources.getString("test.base", locale, null);
        resources.getString("test.specific", locale, null);
        resources.getString("test.inherit", locale, null);
        resources.getString("test.message", locale, null);

        // Accumulate all of the keys that are actually returned
        List results = new ArrayList();
        Iterator keys = resources.getKeys();
        while (keys.hasNext()) {
            results.add(keys.next());
        }

        // Validate the keys that were actually returned
        assertEquals(4, results.size());
        assertTrue(results.contains("test.base"));
        assertTrue(results.contains("test.specific"));
        assertTrue(results.contains("test.inherit"));
        assertTrue(results.contains("test.message"));

    }


    // Test retrieving missing resource keys
    public void testMissing() throws Exception {

        Locale locale = new Locale("en","");
        try {
            resources.setReturnNull(false);
            String value = resources.getString("test.missing", locale, null);
            fail("Should have thrown ResourcesException");
        } catch (ResourcesKeyException e) {
            ; // Expected result
        }
        try {
            resources.setReturnNull(true);
            String value = resources.getString("test.missing", locale, null);
            assertTrue("Should have returned null", value == null);
        } catch (ResourcesException e) {
            fail("Should have returned null");
        }

    }


    // Test the characteristics of a newly created instance
    public void testPristine() {
        assertNotNull(resources);
        assertEquals("Correct name", NAME, resources.getName());
        if (resources instanceof TestResources) {
            assertTrue("Resources initialized",
                       ((TestResources) resources).isInitialized());
        }
    }


    // Test that releasing should destroy instances
    public void testRelease() throws Exception {
        factory.release();
        if (resources instanceof TestResources) {
            assertTrue("Resources destroyed",
                       !((TestResources) resources).isInitialized());
        }
    }


    // Test retrieving Locale-specific Strings explicitly
    public void testSpecific() throws Exception {

        Locale locale = null;

        locale = new Locale("en","");
        assertEquals("en specific value",
                     "[test] SPECIFIC",
                     resources.getString("test.specific", locale, null));

        locale = new Locale("en", "US");
        assertEquals("en_US specific value",
                     "[test] SPECIFIC",
                     resources.getString("test.specific", locale, null));

        locale = new Locale("fr","");
        assertEquals("fr specific value",
                     "[test] SPECIFIC",
                     resources.getString("test.specific", locale, null));

    }


}
